//////////////////////////////////////////////////////////////////////////////
///
///  @file utils.h
///  
///  @author ag  @date   Sep 5, 2017
//////////////////////////////////////////////////////////////////////////////


#ifndef _EXAMPLES_PROJECTSTARTER_UTILS_H_
#define _EXAMPLES_PROJECTSTARTER_UTILS_H_

#include "capd/intervals/lib.h"
#include "capd/capdlib.h"
#include <iostream>
#include "capd/covrel/HSet2D.h"
#include "capd/dynsys/DynSysMap.h"
using namespace std;
using namespace capd;			

typedef capd::covrel::HSet2D<DMatrix, IMatrix> HSet2D;

//////////////////////////////////////////////////////////////////////////////
///	Some auxiliary functions
//////////////////////////////////////////////////////////////////////////////

// modulo pi for intervals
capd::interval modPi(capd::interval x);

// modulo 2 pi for intervals
capd::interval mod2Pi(capd::interval x);

// modPi for first variable and mod2Pi for the third
// as the phase space for Hyperion mod
IVector modXZ(IVector x);

// trace for interval matrices 2x2
capd::interval Tr(capd::IMatrix Dp);

// determinant for interval matrices 2x2
capd::interval Det(capd::IMatrix Dp);	

// determinant of characteristic polynomial	
// for interval matrices 2x2
interval Delta(IMatrix Dp);	

// cuts the third (or further) coordinate(s) of an interval vector
IVector cut(IVector x);		
IMatrix cut(IMatrix M);	
			
//expands an interval 2-vector to 3-vector with zero
IVector expand(IVector x);	
IMatrix expand(IMatrix M);	

//number in the middle of an interval
double srodek(const interval & x);
	

/////////////////////////////////////////////////////////////////////////
/// SecMap class
/////////////////////////////////////////////////////////////////////////

// 2D Poincare Map image of an interval vector on the z=0 section 
// with modPi on the first variable
class SecMap	
{
public:
	IPoincareMap *P;
	SecMap(IPoincareMap &Q) {P=&Q;}	
	SecMap(){};						
	
	IVector image(const IVector &x,IMatrix &DP) const;
	IVector image(const IVector &x) const;
	IMatrix derivative(const IVector &x) const;
	
	IVector operator()(const IVector &x,IMatrix &DP) const {return image(x,DP);}
	IVector operator()(const IVector &x) const {return image(x);}
	IMatrix operator[](const IVector &x) const {return derivative(x);}
};

/////////////////////////////////////////////////////////////////////////
/// PoincNaOsiSym class
/////////////////////////////////////////////////////////////////////////

// Defines the selfmap of the {x=pi/2} line as 
// y -> P(pi/2, y, 0)[0] - pi/2
// its zeroes are the stationary points on the {x=pi/2} line.
class PoincNaOsiSym{
  public: 
  SecMap *P;										
  PoincNaOsiSym( SecMap &pP){P=&pP;};				
	PoincNaOsiSym(){};

  interval image(const interval &x,interval &DP) const;		
  interval image(const interval &x) const;					
  interval derivative(const interval &x) const;				
	
  interval operator()(const interval &x,interval &DP) const {return image(x,DP);}		
  interval operator()(const interval  &x) const {return image(x);}
  interval operator[](const interval &x) const {return derivative(x);}
};

//////////////////////////////////////////////////////////////////////////////
///	Covering auxiliary functions
//////////////////////////////////////////////////////////////////////////////

// lewyBrzeg
// Left face of an interval vector at some coordinate 
IVector lewyBrzeg(const IVector &x, const int coord);

// prawyBrzeg
// Right face of an interval vector at some coordinate 
IVector prawyBrzeg(const IVector &x, const int coord);

// obrazPodzielony
// grids the interval vector N1 and calculates the image via Poincare map,
// in some affine coordinates (M1,p1) of N1 and image (M2, p2)
IVector obrazPodzielony(		
IPoincareMap &P,
const IVector &N1, 
const IMatrix &M1, 
const IVector &p1, 
const IMatrix &M2, 
const IVector &p2,
int podzialH,
int podzialV);

// czyNakrywa
// checks if interval vector obrazX1 lies properly on X2:
// left side to the left of X2, right side to the right of X2,
// and intersection lies beween the top and bottom of X2
bool czyNakrywa(const IVector &obrazX1, const IVector &X2);

//////////////////////////////////////////////////////////////////////////////
///	mojUklad structure
//////////////////////////////////////////////////////////////////////////////

// stores the system modelling the inner rotation of Hyperion
struct mojUklad
{
int order;
IMap vf;
ICoordinateSection section;
IOdeSolver solver;
IPoincareMap P;
int iteracja;

mojUklad() :													// constructor, first iteration
order(20),
vf("par:w2,e,e2,e3;var:x,y,z;fun:y,-(w2*(1+e*cos(z))^3 * sin(2*(x-z)))/e2,(1+e*cos(z))^2/e3;"),		// vector field
section(3,2,2*interval::pi()),																		// z=2 pi section
solver(vf,order),
P(solver,section)
{;
interval epsilon=(interval(1)/interval(10));													// Hyperion parameters
interval epsilon2 = interval(2)*power(interval(1)-power(epsilon,interval(2)),interval(3));
interval epsilon3 = power(interval(1)-power(epsilon,interval(2)),(interval(3)/interval(2)));
interval omega2=power((interval(89)/interval(100)),interval(2));
vf.setParameter("e",epsilon);													
vf.setParameter("e2",epsilon2); //(2*(1-e^2)^3)
vf.setParameter("e3",epsilon3); //(1-e^2)^(3/2)
vf.setParameter("w2",omega2);
}

mojUklad(const int &iter) :										// constructor, iteration = iter
order(20),
iteracja(iter),
vf("par:w2,e,e2,e3,pm;var:x,y,z;fun:pm*y,-(w2*(1+e*cos(z))^3 * sin(2*(x-z)))/e2,(1+e*cos(z))^2/e3;"),
section(3,2,iter*2*interval::pi()),								// z = iter * 2 pi section
solver(vf,order),
P(solver,section)
{;
int plusminus = (iteracja > 0) - (iteracja < 0);				// sign, for negative iterations
interval epsilon=(interval(1)/interval(10));					// Hyperion parameters
interval epsilon2 = plusminus*interval(2)*power(interval(1)-power(epsilon,interval(2)),interval(3));
interval epsilon3 = plusminus*power(interval(1)-power(epsilon,interval(2)),(interval(3)/interval(2)));
interval omega2=power((interval(89)/interval(100)),interval(2));
vf.setParameter("e",epsilon);
vf.setParameter("e2",epsilon2); //(2*(1-e^2)^3)
vf.setParameter("e3",epsilon3); //(1-e^2)^(3/2)
vf.setParameter("w2",omega2);
vf.setParameter("pm",plusminus);
}

// looks for a stationary point on (x=pi/2) line with the starting point z0
// stores eingenvector matrix in Mv
IVector pStNaPi2(const interval &z0, IMatrix &Mv);

// checks covering relation for the h-set of the form 
// p1 + M1*N1 to the h-set p2 + M2*N2 via Hyperion Poincare map
bool covers(
const IVector &N1, 
const IMatrix &M1, 
const IVector &p1, 
const IVector &N2, 
const IMatrix &M2, 
const IVector &p2);

// checks covering relation for the h-set of the form 
// p1 + M1*N1 to the h-set p2 + M2*N2 via Hyperion Poincare map
// with griding podzialH x podzialV
bool covers(
const IVector &N1, 
const IMatrix &M1, 
const IVector &p1, 
const IVector &N2, 
const IMatrix &M2, 
const IVector &p2,
int podzialH,
int podzialV);

};


#endif //_EXAMPLES_PROJECTSTARTER_UTILS_H_


